@@ -1,7 +1,7 @@
 #!/usr/bin/perl -w
 #
 # Virtual Vacation 4.0
-# $Revision: 893 $
+# $Revision: 1074 $
 # Originally by Mischa Peters <mischa at high5 dot net>
 #
 # Copyright (c) 2002 - 2005 High5!
@@ -213,7 +213,7 @@ if($test_mode == 1) {
     if($syslog == 1) {
         my $syslog_appender = Log::Log4perl::Appender->new(
             'Log::Dispatch::Syslog',
-            Facility => 'mail',
+            facility => 'mail',
         );
         $logger->add_appender($syslog_appender);
     }
