@@ -9,14 +9,14 @@
  * Further details on the project are available at : 
  *     http://www.postfixadmin.com or http://postfixadmin.sf.net 
  * 
- * @version $Id: functions.inc.php 999 2011-03-13 20:36:24Z christian_boltz $ 
+ * @version $Id: functions.inc.php 1335 2012-01-15 12:10:59Z christian_boltz $ 
  * @license GNU GPL v2 or later. 
  * 
  * File: functions.inc.php
  * Contains re-usable code.
  */
 
-$version = '2.3.3';
+$version = '2.3.5';
 
 /**
  * check_session
@@ -1107,9 +1107,30 @@ function encode_header ($string, $default_charset = "utf-8")
 // Action: Generates a random password
 // Call: generate_password ()
 //
-function generate_password ()
-{
-    $password = substr (md5 (mt_rand ()), 0, 8);
+function generate_password () {
+    global $CONF;
+
+    //check that password length is sensible
+    $length = (int) $CONF['min_password_length'];
+    if ($length < 5 || $length > 32) {
+  	    $length = 8;
+    }
+
+    // define possible characters
+    $possible = "2345678923456789abcdefghijkmnpqrstuvwxyzABCDEFGHIJKLMNPQRSTUVWXYZ"; # skip 0 and 1 to avoid confusion with O and l
+
+    // add random characters to $password until $length is reached
+    $password = "";
+    while (strlen($password) < $length) {
+        // pick a random character from the possible ones
+        $char = substr($possible, mt_rand(0, strlen($possible)-1), 1);
+
+        // we don't want this character if it's already in the password
+        if (!strstr($password, $char)) {
+            $password .= $char;
+        }
+    }
+
     return $password;
 }
 
@@ -1168,8 +1189,9 @@ function pacrypt ($pw, $pw_db="")
     // this is apparently useful for pam_mysql etc.
     elseif ($CONF['encrypt'] == 'mysql_encrypt')
     {
+        $pw = escape_string($pw);
         if ($pw_db!="") {
-            $salt=substr($pw_db,0,2);
+            $salt=escape_string(substr($pw_db,0,2));
             $res=db_query("SELECT ENCRYPT('".$pw."','".$salt."');");
         } else {
             $res=db_query("SELECT ENCRYPT('".$pw."');");
@@ -1213,7 +1235,8 @@ function pacrypt ($pw, $pw_db="")
         # Use proc_open call to avoid safe_mode problems and to prevent showing plain password in process table
         $spec = array(
             0 => array("pipe", "r"), // stdin
-            1 => array("pipe", "w") // stdout
+            1 => array("pipe", "w"), // stdout
+            2 => array("pipe", "w"), // stderr
         );
 
         $pipe = proc_open("$dovecotpw '-s' $method", $spec, $pipes);
@@ -1229,10 +1252,18 @@ function pacrypt ($pw, $pw_db="")
 
             // Read hash from pipe stdout
             $password = fread($pipes[1], "200");
-    		fclose($pipes[1]);
+
+            if ( !preg_match('/^\{' . $method . '\}/', $password)) {
+                $stderr_output = stream_get_contents($pipes[2]);
+                error_log('dovecotpw password encryption failed.');
+                error_log('STDERR output: ' . $stderr_output);
+                die("can't encrypt password with dovecotpw, see error log for details"); 
+            }
+
+            fclose($pipes[1]);
+            fclose($pipes[2]);
             proc_close($pipe);
 
-            if ( !preg_match('/^\{' . $method . '\}/', $password)) { die("can't encrypt password with dovecotpw"); }
             $password = trim(str_replace('{' . $method . '}', '', $password));
         }
     }
@@ -1325,6 +1356,7 @@ function create_salt ()
     return $salt;
 }
 
+/**/ if (!function_exists('hex2bin')) { # PHP around 5.3.8 includes hex2bin as native function - http://php.net/hex2bin
 function hex2bin ($str)
 {
     $len = strlen ($str);
@@ -1336,6 +1368,7 @@ function hex2bin ($str)
     }
     return $nstr;
 }
+/**/ }
 
 function to64 ($v, $n)
 {
@@ -2319,7 +2352,7 @@ function create_admin($fUsername, $fPassword, $fPassword2, $fDomains, $no_genera
             {
                 for ($i = 0; $i < sizeof ($fDomains); $i++)
                 {
-                    $domain = $fDomains[$i];
+                    $domain = escape_string($fDomains[$i]);
                     $result = db_query ("INSERT INTO " . table_by_key ('domain_admins') . " (username,domain,created) VALUES ('$fUsername','$domain',NOW())");
                 }
             }
